#info Theli-et's Pseudo Kleinian (Scale 1 JuliaBox + Something (here a Menger Sponge)).
#include "DE-Raytracer.frag"
#include "MathUtils.frag"
#group PseudoKleinian

#define USE_INF_NORM

// Maximum iterations
uniform int MI; slider[0,5,20]

// Size
uniform float Size; slider[0,1,2]

// Cubic fold Size
uniform vec3 CSize; slider[(0,0,0),(1,1,1),(2,2,2)]

// Julia constant
uniform vec3 C; slider[(-2,-2,-2),(0,0,0),(2,2,2)]

// Basic shape DE Offset
uniform float DEoffset; slider[0,0,0.01]

// Basic shape Translation
uniform vec3 Offset; slider[(-2,-2,-2),(0,0,0),(2,2,2)]

//Basic shape
// Menger Number of iterations.
uniform int MnIterations;  slider[0,2,20]

// Menger Scale parameter. A perfect Menger is 3.0
uniform float MnScale; slider[0.00,3.0,4.00]

// Menger Scaling center
uniform vec3 MnOffset; slider[(0,0,0),(1,1,1),(2,2,2)]

void init() {
}

float Menger(vec3 z)
{
	float r;
	
	int n = 0;
	// Fold
	z = abs(z);
	if (z.x<z.y){ z.xy = z.yx;}
	if (z.x<z.z){ z.xz = z.zx;}
	if (z.y<z.z){ z.yz = z.zy;}
	if (z.z<1./3.){ z.z -=2.*( z.z-1./3.);}
	
while (n < MnIterations && dot(z,z)<100.0) {
		
		z=MnScale* (z-MnOffset)+MnOffset;
		
		// Fold
		z = abs(z);
		if (z.x<z.y){ z.xy = z.yx;}
		if (z.x< z.z){ z.xz = z.zx;}
		if (z.y<z.z){ z.yz = z.zy;}
		if (z.z<1./3.*MnOffset.z){ z.z -=2.*( z.z-1./3.*MnOffset.z);}
		
		r = dot(z-MnOffset, z-MnOffset);
		orbitTrap = min(orbitTrap, abs(vec4(z,r)));
		
		n++;
	}
	
	//return (length(z)-sqrt(3.) ) * pow(Scale, float(-n));
	return (z.x-MnOffset) * pow(MnScale, float(-n));
}

float Thing2(vec3 p){
//Just scale=1 Julia box
#ifdef USE_INF_NORM   
	vec3 p1=abs(p);
	float r2=max(p1.x,max(p1.y,p1.z));
#else
	float r2=dot(p,p);
#endif
	float DEfactor=1.;
   
	for(int i=0;i<MI && r2<60.;i++){
		p=2.*clamp(p, -CSize, CSize)-p;
      
		r2=dot(p,p);
		//orbitTrap = min(orbitTrap, abs(vec4(p,r2)));
		float k=max(Size/r2,1.);
		p*=k;DEfactor*=k;
      
		p+=C;
#ifdef USE_INF_NORM   
		p1=abs(p);
		r2=max(p1.x,max(p1.y,p1.z));
#else
		 r2=dot(p,p);
#endif
		orbitTrap = min(orbitTrap, abs(vec4(p,r2)));
	}
	//Call basic shape and scale its DE
	return abs(0.5*Menger(p-Offset)/DEfactor-DEoffset);
}

float DE(vec3 p){
	return  Thing2(p);//RoundBox(p, CSize, Offset);
}
